Package: roxigraph
Title: 'RDF' and 'SPARQL' for R using 'Oxigraph'
Version: 0.1.2
Authors@R: c(
    person("Carl", "Boettiger", , "cboettig@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-1642-628X")),
    person("Oxigraph Contributors", role = "cph",
           comment = "Oxigraph Rust library"),
    person("Authors of the dependency Rust crates", role = "aut",
           comment = "see inst/AUTHORS file")
    )
Description: Provides 'RDF' storage and 'SPARQL' 1.1 query capabilities by wrapping
    the 'Oxigraph' graph database library <https://github.com/oxigraph/oxigraph>. 
    Supports in-memory and persistent ('RocksDB') storage, multiple 'RDF' 
    serialization formats ('Turtle', 'N-Triples', 'RDF-XML', 'N-Quads', 'TriG'), 
    and full 'SPARQL' 1.1 Query and Update support. Built using the 'extendr' 
    framework for 'Rust'-R bindings.
License: MIT + file LICENSE
URL: https://github.com/cboettig/roxigraph
BugReports: https://github.com/cboettig/roxigraph/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Config/rextendr/version: 0.4.2
SystemRequirements: Cargo (Rust's package manager), rustc >= 1.75.0,
        libclang-dev (for RocksDB bindings), libsnappy-dev (for RocksDB
        compression)
Depends: R (>= 4.2)
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, spelling
Config/testthat/edition: 3
VignetteBuilder: knitr
Language: en-US
NeedsCompilation: yes
Packaged: 2026-05-04 04:51:17 UTC; cboettig
Author: Carl Boettiger [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1642-628X>),
  Oxigraph Contributors [cph] (Oxigraph Rust library),
  Authors of the dependency Rust crates [aut] (see inst/AUTHORS file)
Maintainer: Carl Boettiger <cboettig@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-04 16:20:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-04 22:53:09 UTC; unix
Archs: roxigraph.so.dSYM

# Generated by roxygen2: do not edit by hand

export(rdf_add)
export(rdf_load)
export(rdf_load_file)
export(rdf_remove)
export(rdf_serialize)
export(rdf_size)
export(rdf_store)
export(rdf_store_dump)
export(rdf_store_insert)
export(rdf_store_load)
export(rdf_store_new)
export(rdf_store_open)
export(rdf_store_query)
export(rdf_store_remove)
export(rdf_store_size)
export(rdf_store_update)
export(sparql_query)
export(sparql_update)
useDynLib(roxigraph, .registration = TRUE)

---
output: github_document
---

<!-- README.md is generated from README.Rmd. Please edit that file -->



# roxigraph <img src="man/figures/logo.png" align="right" height="139" alt="" />

<!-- badges: start -->
[![R-CMD-check](https://github.com/cboettig/roxigraph/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/cboettig/roxigraph/actions/workflows/R-CMD-check.yaml)
[![CRAN status](https://www.r-pkg.org/badges/version/roxigraph)](https://CRAN.R-project.org/package=roxigraph)
[![r-universe](https://cboettig.r-universe.dev/badges/roxigraph)](https://cboettig.r-universe.dev/roxigraph)
[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)
<!-- badges: end -->

**roxigraph** provides RDF storage and SPARQL 1.1 query capabilities for R by wrapping the [Oxigraph](https://github.com/oxigraph/oxigraph) graph database library. Built using the [extendr](https://extendr.rs/) framework for high-performance Rust-R bindings.

## Features

- 🚀 **High Performance**: Rust-powered RDF storage with RocksDB backend
- 💾 **Flexible Storage**: In-memory or persistent on-disk databases
- 📄 **Multiple Formats**: Turtle, N-Triples, RDF-XML, N-Quads, TriG
- 🔍 **Full SPARQL 1.1**: Query and Update support

## Installation

### Windows / macOS (recommended)

Install pre-built binaries from [r-universe](https://cboettig.r-universe.dev/roxigraph):

``` r
install.packages("roxigraph", repos = c("https://cboettig.r-universe.dev", "https://cloud.r-project.org"))
```

### Linux (Ubuntu)

For binary installation on Ubuntu (no Rust required):

``` r
# Set up r-universe binary repo for Linux
repo <- sprintf(
  "https://cboettig.r-universe.dev/bin/linux/noble-%s/%s/",
  R.version$arch, substr(getRversion(), 1, 3)
)
install.packages("roxigraph", repos = c(repo, "https://cloud.r-project.org"))
```

### From source (requires Rust)


``` r
# install.packages("pak")
pak::pak("cboettig/roxigraph")
```

Building from source requires:
- **Rust** toolchain (rustc >= 1.65, cargo)
- **libclang-dev** (for RocksDB bindings on Linux)

On Ubuntu/Debian:
```bash
# Install Rust
curl --proto '=https' --tlsv1.2 -sSf https://sh.rustup.rs | sh

# Install libclang
sudo apt-get install libclang-dev
```

## Quick Start


``` r
library(roxigraph)

# Create an in-memory store
store <- rdf_store()

# Load some RDF data (Turtle format)
rdf_load(store, '
@prefix foaf: <http://xmlns.com/foaf/0.1/> .
@prefix ex: <http://example.org/> .

ex:alice foaf:name "Alice" ;
         foaf:knows ex:bob .
ex:bob foaf:name "Bob" .
', format = "turtle")

# Query with SPARQL
sparql_query(store, "
  PREFIX foaf: <http://xmlns.com/foaf/0.1/>
  SELECT ?name WHERE { ?person foaf:name ?name }
  ORDER BY ?name
")
#>      name
#> 1 "Alice"
#> 2   "Bob"

# Check store size
rdf_size(store)
#> [1] 3
```

## Persistent Storage

For data that should persist across R sessions:


``` r
# Create a persistent store
store <- rdf_store("/path/to/my/database")

# Data is automatically persisted to disk
rdf_load(store, data, format = "turtle")
```

## SPARQL Update

Modify your data using SPARQL Update:


``` r
store <- rdf_store()

# Insert data
sparql_update(store, "
  INSERT DATA {
    <http://example.org/alice> <http://xmlns.com/foaf/0.1/name> 'Alice' .
  }
")

# Query the inserted data
sparql_query(store, "SELECT * WHERE { ?s ?p ?o }")
#>         o                                p                          s
#> 1 "Alice" <http://xmlns.com/foaf/0.1/name> <http://example.org/alice>
```

## Serialization

Export your RDF data to various formats:


``` r
store <- rdf_store()
rdf_load(store, '<http://ex.org/s> <http://ex.org/p> "hello" .', format = "ntriples")

# Serialize to N-Quads
cat(rdf_serialize(store, format = "nquads"))
#> <http://ex.org/s> <http://ex.org/p> "hello" .
```

## Learn More

- [Package documentation](https://cboettig.github.io/roxigraph/)
- [Oxigraph](https://github.com/oxigraph/oxigraph) - the underlying Rust library
- [SPARQL 1.1 specification](https://www.w3.org/TR/sparql11-query/)

## License
MIT © Carl Boettiger

Oxigraph is dual-licensed under MIT/Apache-2.0.

# Generated by extendr: Do not edit by hand

# nolint start

#
# This file was created with the following call:
#   .Call("wrap__make_roxigraph_wrappers", use_symbols = TRUE, package_name = "roxigraph")

#' @usage NULL
#' @useDynLib roxigraph, .registration = TRUE
NULL

#' Create a new in-memory RDF store
#' 
#' @section Arguments:
#' This function takes no arguments.
#' 
#' @return Store index (integer handle)
#' @keywords internal
#' @export
#' @examples
#' store <- rdf_store_new()
#' rdf_store_size(store)
rdf_store_new <- function() .Call(wrap__rdf_store_new)

#' Open or create a persistent RDF store at the given path
#' @param path Path to the store directory
#' @return Store index (integer handle)
#' @keywords internal
#' @export
#' @examples
#' \donttest{
#' if (.Platform$OS.type != "windows") {
#'   store <- rdf_store_open(file.path(tempdir(), "roxigraph_test"))
#' }
#' }
rdf_store_open <- function(path) .Call(wrap__rdf_store_open, path)

#' Get the number of quads in the store
#' @param store_idx Store index
#' @return The number of quads
#' @keywords internal
#' @export
#' @examples
#' store <- rdf_store_new()
#' rdf_store_size(store)
rdf_store_size <- function(store_idx) .Call(wrap__rdf_store_size, store_idx)

#' Load RDF data into the store
#' @param store_idx Store index
#' @param data RDF data as a string
#' @param format RDF format: "turtle", "ntriples", "rdfxml", "nquads", "trig"
#' @param base_iri Optional base IRI for relative URIs
#' @return No return value, called for side effects (loading data into the store)
#' @keywords internal
#' @export
#' @examples
#' store <- rdf_store_new()
#' rdf_store_load(store, '<http://example.org/s> <http://example.org/p> "v" .', "ntriples", NULL)
rdf_store_load <- function(store_idx, data, format, base_iri) invisible(.Call(wrap__rdf_store_load, store_idx, data, format, base_iri))

#' Serialize the store contents to a string
#' @param store_idx Store index
#' @param format RDF format: "turtle", "ntriples", "rdfxml", "nquads", "trig"
#' @return The serialized RDF data
#' @keywords internal
#' @export
#' @examples
#' store <- rdf_store_new()
#' rdf_store_load(store, '<http://example.org/s> <http://example.org/p> "v" .', "ntriples", NULL)
#' rdf_store_dump(store, "turtle")
rdf_store_dump <- function(store_idx, format) .Call(wrap__rdf_store_dump, store_idx, format)

#' Execute a SPARQL query and return results as a data frame
#' @param store_idx Store index
#' @param query SPARQL query string
#' @return Query results as a data frame (for SELECT) or logical (for ASK)
#' @keywords internal
#' @export
#' @examples
#' store <- rdf_store_new()
#' rdf_store_load(store, '<http://example.org/s> <http://example.org/p> "v" .', "ntriples", NULL)
#' rdf_store_query(store, "SELECT * WHERE { ?s ?p ?o }")
rdf_store_query <- function(store_idx, query) .Call(wrap__rdf_store_query, store_idx, query)

#' Insert a triple into the store
#' @param store_idx Store index
#' @param subject Subject IRI (e.g., `"<http://example.org/s>"`) or blank node ("_:b1")
#' @param predicate Predicate IRI (e.g., `"<http://example.org/p>"`)
#' @param object Object (IRI, blank node, or literal with quotes e.g., "\"value\"")
#' @param graph Optional graph name IRI
#' @return No return value, called for side effects (inserting triples into the store)
#' @keywords internal
#' @export
#' @examples
#' store <- rdf_store_new()
#' rdf_store_insert(store, "<http://example.org/s>", "<http://example.org/p>", '"val"', NULL)
rdf_store_insert <- function(store_idx, subject, predicate, object, graph) invisible(.Call(wrap__rdf_store_insert, store_idx, subject, predicate, object, graph))

#' Remove a triple from the store
#' @param store_idx Store index
#' @param subject Subject IRI or blank node
#' @param predicate Predicate IRI
#' @param object Object
#' @param graph Optional graph name IRI
#' @return No return value, called for side effects (removing triples from the store)
#' @keywords internal
#' @export
#' @examples
#' store <- rdf_store_new()
#' rdf_store_insert(store, "<http://example.org/s>", "<http://example.org/p>", '"val"', NULL)
#' rdf_store_remove(store, "<http://example.org/s>", "<http://example.org/p>", '"val"', NULL)
rdf_store_remove <- function(store_idx, subject, predicate, object, graph) invisible(.Call(wrap__rdf_store_remove, store_idx, subject, predicate, object, graph))

#' Execute a SPARQL UPDATE query
#' @param store_idx Store index
#' @param update SPARQL UPDATE query string
#' @return No return value, called for side effects (executing SPARQL UPDATE queries)
#' @keywords internal
#' @export
#' @examples
#' store <- rdf_store_new()
#' rdf_store_update(store, "INSERT DATA { <http://example.org/s> <http://example.org/p> 'val' }")
rdf_store_update <- function(store_idx, update) invisible(.Call(wrap__rdf_store_update, store_idx, update))


# nolint end

#' Create an RDF Store
#'
#' Creates a new RDF store, either in-memory or backed by persistent storage.
#'
#' @param path Optional path for persistent storage. If NULL (default), creates
#'   an in-memory store.
#' @return An RDF store handle (integer)
#' @export
#' @examples
#' # In-memory store
#' store <- rdf_store()
#'
#' # Persistent store (not supported on Windows)
#' \donttest{
#' if (.Platform$OS.type != "windows") {
#'   store <- rdf_store(file.path(tempdir(), "my_store"))
#' }
#' }
rdf_store <- function(path = NULL) {
    if (is.null(path)) {
        rdf_store_new()
    } else {
        rdf_store_open(path)
    }
}

#' Execute a SPARQL Query
#'
#' Executes a SPARQL query against the RDF store.
#'
#' @param store An RDF store handle
#' @param query A SPARQL query string
#' @return For SELECT queries, a data.frame with results. For ASK queries, a logical.
#'   For CONSTRUCT/DESCRIBE queries, a data.frame with subject, predicate, object columns.
#' @export
#' @examples
#' store <- rdf_store()
#' rdf_load(store, '<http://example.org/s> <http://example.org/p> "hello" .', format = "ntriples")
#' sparql_query(store, "SELECT * WHERE { ?s ?p ?o }")
sparql_query <- function(store, query) {
    rdf_store_query(store, query)
}

#' Load RDF Data
#'
#' Loads RDF data into the store from a string.
#'
#' @param store An RDF store handle
#' @param data RDF data as a character string
#' @param format RDF format: "turtle", "ntriples", "rdfxml", "nquads", or "trig"
#' @param base_iri Optional base IRI for resolving relative URIs
#' @return Invisibly returns NULL
#' @export
#' @examples
#' store <- rdf_store()
#' rdf_load(store, '<http://example.org/s> <http://example.org/p> "value" .', format = "ntriples")
rdf_load <- function(store, data, format = "turtle", base_iri = NULL) {
    rdf_store_load(store, data, format, base_iri)
    invisible(NULL)
}

#' Load RDF from File
#'
#' Loads RDF data into the store from a file.
#'
#' @param store An RDF store handle
#' @param file Path to the RDF file
#' @param format RDF format. If NULL, guessed from file extension.
#' @param base_iri Optional base IRI for resolving relative URIs
#' @return Invisibly returns NULL
#' @export
#' @examples
#' store <- rdf_store()
#' # Create a temporary RDF file
#' tmp <- tempfile(fileext = ".nt")
#' writeLines('<http://example.org/s> <http://example.org/p> "value" .', tmp)
#' rdf_load_file(store, tmp)
#' rdf_size(store)
rdf_load_file <- function(store, file, format = NULL, base_iri = NULL) {
    if (is.null(format)) {
        ext <- tolower(tools::file_ext(file))
        format <- switch(
            ext,
            "ttl" = "turtle",
            "nt" = "ntriples",
            "rdf" = "rdfxml",
            "xml" = "rdfxml",
            "nq" = "nquads",
            "trig" = "trig",
            stop("Cannot guess format from extension: ", ext)
        )
    }
    data <- paste(readLines(file, warn = FALSE), collapse = "\n")
    rdf_store_load(store, data, format, base_iri)
    invisible(NULL)
}

#' Serialize RDF Data
#'
#' Serializes the store contents to a string.
#'
#' @param store An RDF store handle
#' @param format RDF format: "turtle", "ntriples", "rdfxml", "nquads", or "trig"
#' @return The serialized RDF data as a character string
#' @export
#' @examples
#' store <- rdf_store()
#' rdf_load(store, '<http://example.org/s> <http://example.org/p> "value" .', format = "ntriples")
#' rdf_serialize(store, format = "turtle")
rdf_serialize <- function(store, format = "turtle") {
    rdf_store_dump(store, format)
}

#' Get Store Size
#'
#' Returns the number of quads (triples) in the store.
#'
#' @param store An RDF store handle
#' @return The number of quads as an integer
#' @export
#' @examples
#' store <- rdf_store()
#' rdf_size(store)
rdf_size <- function(store) {
    rdf_store_size(store)
}

#' Add a Triple
#'
#' Adds a single triple to the store.
#'
#' @param store An RDF store handle
#' @param subject Subject IRI (e.g., `"<http://example.org/s>"`) or blank node ("_:b1")
#' @param predicate Predicate IRI (e.g., `"<http://example.org/p>"`)
#' @param object Object: IRI, blank node, or literal (e.g., '"value"')
#' @param graph Optional named graph IRI
#' @return Invisibly returns NULL
#' @export
#' @examples
#' store <- rdf_store()
#' rdf_add(store, "<http://example.org/s>", "<http://example.org/p>", '"hello"')
#' rdf_size(store)
rdf_add <- function(store, subject, predicate, object, graph = NULL) {
    rdf_store_insert(store, subject, predicate, object, graph)
    invisible(NULL)
}

#' Remove a Triple
#'
#' Removes a single triple from the store.
#'
#' @param store An RDF store handle
#' @param subject Subject IRI or blank node
#' @param predicate Predicate IRI
#' @param object Object: IRI, blank node, or literal
#' @param graph Optional named graph IRI
#' @return Invisibly returns NULL
#' @export
#' @examples
#' store <- rdf_store()
#' rdf_add(store, "<http://example.org/s>", "<http://example.org/p>", '"hello"')
#' rdf_remove(store, "<http://example.org/s>", "<http://example.org/p>", '"hello"')
#' rdf_size(store)
rdf_remove <- function(store, subject, predicate, object, graph = NULL) {
    rdf_store_remove(store, subject, predicate, object, graph)
    invisible(NULL)
}

#' Execute SPARQL Update
#'
#' Executes a SPARQL UPDATE query to modify the store.
#'
#' @param store An RDF store handle
#' @param update A SPARQL UPDATE query string
#' @return Invisibly returns NULL
#' @export
#' @examples
#' store <- rdf_store()
#' sparql_update(store, "INSERT DATA { <http://example.org/s> <http://example.org/p> 'value' }")
sparql_update <- function(store, update) {
    rdf_store_update(store, update)
    invisible(NULL)
}

test_that("rdf_store creates in-memory store", {
    store <- rdf_store()
    expect_type(store, "integer")
    expect_equal(rdf_size(store), 0)
})

test_that("rdf_store creates persistent store", {
    skip_on_cran() # Requires file system access
    skip_on_os("windows") # RocksDB disabled on Windows

    tmp_dir <- tempfile("roxigraph_test_")
    store <- rdf_store(tmp_dir)
    expect_type(store, "integer")

    # Add data
    rdf_load(
        store,
        '<http://example.org/s> <http://example.org/p> "test" .',
        format = "ntriples"
    )
    expect_equal(rdf_size(store), 1)

    # Cleanup
    unlink(tmp_dir, recursive = TRUE)
})

test_that("rdf_size returns correct count", {
    store <- rdf_store()
    expect_equal(rdf_size(store), 0)

    rdf_load(
        store,
        '<http://ex.org/a> <http://ex.org/b> "1" .',
        format = "ntriples"
    )
    expect_equal(rdf_size(store), 1)

    rdf_load(
        store,
        '<http://ex.org/c> <http://ex.org/d> "2" .',
        format = "ntriples"
    )
    expect_equal(rdf_size(store), 2)
})

test_that("rdf_serialize outputs N-Quads", {
    store <- rdf_store()
    rdf_load(
        store,
        '<http://example.org/s> <http://example.org/p> "value" .',
        format = "ntriples"
    )

    output <- rdf_serialize(store, format = "nquads")

    expect_type(output, "character")
    expect_true(grepl("http://example.org/s", output))
    expect_true(grepl("http://example.org/p", output))
    expect_true(grepl("value", output))
})

test_that("rdf_serialize outputs TriG", {
    store <- rdf_store()
    rdf_load(
        store,
        '<http://example.org/s> <http://example.org/p> "value" .',
        format = "ntriples"
    )

    output <- rdf_serialize(store, format = "trig")

    expect_type(output, "character")
    expect_true(nchar(output) > 0)
})

test_that("rdf_serialize handles empty store", {
    store <- rdf_store()

    output <- rdf_serialize(store, format = "nquads")

    expect_type(output, "character")
    expect_equal(nchar(output), 0)
})

test_that("sparql_query returns data frame for SELECT", {
    store <- rdf_store()
    rdf_load(
        store,
        '<http://ex.org/s> <http://ex.org/p> "value" .',
        format = "ntriples"
    )

    results <- sparql_query(store, "SELECT * WHERE { ?s ?p ?o }")

    expect_s3_class(results, "data.frame")
    expect_true(all(c("s", "p", "o") %in% names(results)))
    expect_equal(nrow(results), 1)
})

test_that("sparql_query returns logical for ASK", {
    store <- rdf_store()
    rdf_load(
        store,
        '<http://ex.org/s> <http://ex.org/p> "value" .',
        format = "ntriples"
    )

    result_true <- sparql_query(store, "ASK { ?s ?p ?o }")
    expect_true(result_true)

    result_false <- sparql_query(store, "ASK { <http://nonexistent> ?p ?o }")
    expect_false(result_false)
})

test_that("sparql_query handles FILTER", {
    store <- rdf_store()
    turtle <- '@prefix ex: <http://example.org/> .
ex:a ex:value "10"^^<http://www.w3.org/2001/XMLSchema#integer> .
ex:b ex:value "20"^^<http://www.w3.org/2001/XMLSchema#integer> .
ex:c ex:value "30"^^<http://www.w3.org/2001/XMLSchema#integer> .'

    rdf_load(store, turtle, format = "turtle")

    results <- sparql_query(
        store,
        "
    PREFIX ex: <http://example.org/>
    SELECT ?s ?v WHERE { ?s ex:value ?v FILTER(?v > 15) }
  "
    )

    expect_equal(nrow(results), 2)
})

test_that("sparql_query handles ORDER BY", {
    store <- rdf_store()
    rdf_load(
        store,
        '
    <http://ex.org/c> <http://ex.org/name> "Charlie" .
    <http://ex.org/a> <http://ex.org/name> "Alice" .
    <http://ex.org/b> <http://ex.org/name> "Bob" .
  ',
        format = "ntriples"
    )

    results <- sparql_query(
        store,
        "SELECT ?n WHERE { ?s <http://ex.org/name> ?n } ORDER BY ?n"
    )

    expect_equal(nrow(results), 3)
    # First result should be Alice (alphabetically first)
    expect_true(grepl("Alice", results$n[1]))
})

test_that("sparql_query handles LIMIT", {
    store <- rdf_store()
    turtle <- '@prefix ex: <http://example.org/> .
ex:a ex:p "1" .
ex:b ex:p "2" .
ex:c ex:p "3" .'

    rdf_load(store, turtle, format = "turtle")

    results <- sparql_query(store, "SELECT * WHERE { ?s ?p ?o } LIMIT 2")
    expect_equal(nrow(results), 2)
})

test_that("sparql_query handles OPTIONAL", {
    store <- rdf_store()
    turtle <- '@prefix ex: <http://example.org/> .
ex:a ex:name "Alice" ; ex:age "30" .
ex:b ex:name "Bob" .'

    rdf_load(store, turtle, format = "turtle")

    results <- sparql_query(
        store,
        "
    PREFIX ex: <http://example.org/>
    SELECT ?name ?age WHERE {
      ?s ex:name ?name .
      OPTIONAL { ?s ex:age ?age }
    }
  "
    )

    expect_equal(nrow(results), 2)
})

test_that("sparql_query handles COUNT", {
    store <- rdf_store()
    rdf_load(
        store,
        '
    <http://ex.org/a> <http://ex.org/p> "1" .
    <http://ex.org/b> <http://ex.org/p> "2" .
    <http://ex.org/c> <http://ex.org/p> "3" .
  ',
        format = "ntriples"
    )

    results <- sparql_query(
        store,
        "SELECT (COUNT(*) as ?count) WHERE { ?s ?p ?o }"
    )

    expect_equal(nrow(results), 1)
    expect_true("count" %in% names(results))
})

test_that("sparql_update INSERT DATA works", {
    store <- rdf_store()

    sparql_update(
        store,
        "INSERT DATA { <http://ex.org/s> <http://ex.org/p> 'value' }"
    )

    expect_equal(rdf_size(store), 1)

    results <- sparql_query(store, "SELECT * WHERE { ?s ?p ?o }")
    expect_equal(nrow(results), 1)
})

test_that("sparql_update DELETE DATA works", {
    store <- rdf_store()
    rdf_load(
        store,
        '<http://ex.org/s> <http://ex.org/p> "value" .',
        format = "ntriples"
    )
    expect_equal(rdf_size(store), 1)

    sparql_update(
        store,
        "DELETE DATA { <http://ex.org/s> <http://ex.org/p> 'value' }"
    )

    expect_equal(rdf_size(store), 0)
})

test_that("sparql_update INSERT WHERE works", {
    store <- rdf_store()
    rdf_load(
        store,
        '<http://ex.org/a> <http://ex.org/name> "Alice" .',
        format = "ntriples"
    )

    sparql_update(
        store,
        "
    INSERT { ?s <http://ex.org/greeting> 'Hello' }
    WHERE { ?s <http://ex.org/name> ?name }
  "
    )

    expect_equal(rdf_size(store), 2)
})

test_that("rdf_add inserts triple", {
    store <- rdf_store()

    rdf_add(store, "<http://ex.org/s>", "<http://ex.org/p>", '"object"')

    expect_equal(rdf_size(store), 1)
})

test_that("rdf_remove deletes triple", {
    store <- rdf_store()
    rdf_add(store, "<http://ex.org/s>", "<http://ex.org/p>", '"object"')
    expect_equal(rdf_size(store), 1)

    rdf_remove(store, "<http://ex.org/s>", "<http://ex.org/p>", '"object"')

    expect_equal(rdf_size(store), 0)
})

test_that("rdf_load parses N-Triples", {
    store <- rdf_store()

    nt_data <- '<http://example.org/s> <http://example.org/p> "hello" .'
    rdf_load(store, nt_data, format = "ntriples")

    expect_equal(rdf_size(store), 1)
})

test_that("rdf_load parses Turtle", {
    store <- rdf_store()

    turtle_data <- '@prefix ex: <http://example.org/> .
ex:subject ex:predicate "object" .'

    rdf_load(store, turtle_data, format = "turtle")
    expect_equal(rdf_size(store), 1)
})

test_that("rdf_load parses Turtle with multiple triples", {
    store <- rdf_store()

    turtle_data <- '@prefix ex: <http://example.org/> .
@prefix foaf: <http://xmlns.com/foaf/0.1/> .

ex:alice foaf:name "Alice" ;
         foaf:knows ex:bob .
ex:bob foaf:name "Bob" .'

    rdf_load(store, turtle_data, format = "turtle")
    expect_equal(rdf_size(store), 3)
})

test_that("rdf_load handles language tags", {
    store <- rdf_store()

    data <- '<http://example.org/s> <http://example.org/label> "Hello"@en .'
    rdf_load(store, data, format = "ntriples")

    results <- sparql_query(store, "SELECT ?o WHERE { ?s ?p ?o }")
    expect_true(grepl("@en", results$o[1]))
})

test_that("rdf_load handles typed literals", {
    store <- rdf_store()

    data <- '<http://example.org/s> <http://example.org/value> "42"^^<http://www.w3.org/2001/XMLSchema#integer> .'
    rdf_load(store, data, format = "ntriples")

    results <- sparql_query(store, "SELECT ?o WHERE { ?s ?p ?o }")
    expect_true(grepl("42", results$o[1]))
})

test_that("rdf_load_file reads from file", {
    store <- rdf_store()

    tmp_file <- tempfile(fileext = ".nt")
    writeLines(
        '<http://example.org/s> <http://example.org/p> "from file" .',
        tmp_file
    )

    rdf_load_file(store, tmp_file)
    expect_equal(rdf_size(store), 1)

    unlink(tmp_file)
})

#' @title roxigraph: RDF and SPARQL for R
#'
#' @description
#' Provides RDF storage and SPARQL 1.1 query capabilities by wrapping the
#' 'Oxigraph' graph database library. Supports in-memory and persistent ('RocksDB')
#' storage, multiple RDF serialization formats, and full SPARQL 1.1 Query and
#' Update support.
#'
#' @section Main Functions:
#' \itemize{
#'   \item \code{\link{rdf_store}}: Create an RDF store
#'   \item \code{\link{rdf_load}}: Load RDF data
#'   \item \code{\link{sparql_query}}: Execute SPARQL queries
#'   \item \code{\link{sparql_update}}: Execute SPARQL updates
#'   \item \code{\link{rdf_serialize}}: Serialize RDF data
#' }
#'
#' @seealso
#' Useful links:
#' \itemize{
#'   \item \url{https://github.com/cboettig/roxigraph}
#'   \item \url{https://github.com/oxigraph/oxigraph}
#'   \item Report bugs at \url{https://github.com/cboettig/roxigraph/issues}
#' }
#'
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @useDynLib roxigraph, .registration = TRUE
## usethis namespace: end
NULL

# Generate inst/AUTHORS from cargo metadata
# This script is called by the cleanup script during R CMD build

if (!require('jsonlite', quietly = TRUE)) {
    install.packages('jsonlite', repos = 'https://cloud.r-project.org')
}

metadata <- jsonlite::fromJSON(pipe("cargo metadata --format-version 1"))
packages <- metadata$packages
stopifnot(is.data.frame(packages))

# Filter out the main package and packages without authors
packages <- subset(
    packages,
    sapply(packages$authors, length) > 0 & name != 'roxigraph'
)

# Format authors (remove email addresses)
authors <- vapply(
    packages$authors,
    function(x) {
        paste(sub(" <.*>", "", x), collapse = ', ')
    },
    character(1)
)

# Create formatted lines
lines <- sprintf(" - %s %s: %s", packages$name, packages$version, authors)

# Create inst directory if needed
dir.create('../../inst', showWarnings = FALSE)

# Write AUTHORS file
footer <- sprintf(
    "\n(This file was auto-generated from 'cargo metadata' on %s)",
    Sys.Date()
)
writeLines(
    c('Authors of vendored cargo crates', lines, footer),
    '../../inst/AUTHORS'
)

message("Generated inst/AUTHORS with ", length(lines), " crate entries")

---
title: "Getting Started with roxigraph"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with roxigraph}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
```

## Introduction

**roxigraph** provides RDF storage and SPARQL query capabilities for R by wrapping the [Oxigraph](https://github.com/oxigraph/oxigraph) graph database. This vignette introduces the core functionality.

## Creating an RDF Store

```{r setup}
library(roxigraph)

# Create an in-memory store
store <- rdf_store()
```

For persistent storage that survives R sessions:

```{r eval=FALSE}
store <- rdf_store("/path/to/database")
```

## Loading RDF Data

roxigraph supports multiple RDF serialization formats:

### Turtle

```{r}
turtle_data <- '
@prefix foaf: <http://xmlns.com/foaf/0.1/> .
@prefix ex: <http://example.org/> .

ex:alice a foaf:Person ;
         foaf:name "Alice" ;
         foaf:age 30 ;
         foaf:knows ex:bob .

ex:bob a foaf:Person ;
       foaf:name "Bob" ;
       foaf:age 25 .
'

rdf_load(store, turtle_data, format = "turtle")
rdf_size(store)
```

### N-Triples

```{r}
store2 <- rdf_store()
nt_data <- '<http://example.org/s> <http://example.org/p> "object" .'
rdf_load(store2, nt_data, format = "ntriples")
```

### From Files

```{r eval=FALSE}
rdf_load_file(store, "data.ttl", format = "turtle")
rdf_load_file(store, "data.nt") # Format guessed from extension
```

## SPARQL Queries

### SELECT Queries

```{r}
# Find all people and their names
results <- sparql_query(store, "
  PREFIX foaf: <http://xmlns.com/foaf/0.1/>
  SELECT ?person ?name
  WHERE {
    ?person a foaf:Person ;
            foaf:name ?name .
  }
")
results
```

### Filtering and Sorting

```{r}
# Find people over 26
sparql_query(store, "
  PREFIX foaf: <http://xmlns.com/foaf/0.1/>
  SELECT ?name ?age
  WHERE {
    ?person foaf:name ?name ;
            foaf:age ?age .
    FILTER(?age > 26)
  }
  ORDER BY DESC(?age)
")
```

### ASK Queries

```{r}
# Check if Alice exists
sparql_query(store, "
  PREFIX ex: <http://example.org/>
  ASK { ex:alice ?p ?o }
")
```

### Aggregation

```{r}
sparql_query(store, "
  PREFIX foaf: <http://xmlns.com/foaf/0.1/>
  SELECT (COUNT(?person) as ?count) (AVG(?age) as ?avg_age)
  WHERE {
    ?person a foaf:Person ;
            foaf:age ?age .
  }
")
```

## Modifying Data

### SPARQL Update

```{r}
# Add new data
sparql_update(store, "
  PREFIX foaf: <http://xmlns.com/foaf/0.1/>
  PREFIX ex: <http://example.org/>

  INSERT DATA {
    ex:carol a foaf:Person ;
             foaf:name 'Carol' ;
             foaf:age 28 .
  }
")

rdf_size(store)
```

### Direct Triple Manipulation

```{r}
# Add a triple
rdf_add(
    store, "<http://example.org/carol>",
    "<http://xmlns.com/foaf/0.1/knows>",
    "<http://example.org/alice>"
)

# Remove a triple
rdf_remove(
    store, "<http://example.org/carol>",
    "<http://xmlns.com/foaf/0.1/knows>",
    "<http://example.org/alice>"
)
```

## Serialization

Export your RDF data:

```{r}
# Export to N-Quads format
output <- rdf_serialize(store, format = "nquads")
cat(substr(output, 1, 500), "...\n")
```

Supported output formats: `nquads`, `trig`, `rdfxml`

## Best Practices

1. **Use prefixes** in SPARQL queries for readability
2. **Choose the right storage**: In-memory for temporary data, persistent for large datasets
3. **Batch operations**: Use `rdf_load()` for bulk data rather than individual `rdf_add()` calls
4. **Close persistent stores**: R will handle cleanup, but explicit management helps with large databases

## Further Reading

- [SPARQL 1.1 Query Language](https://www.w3.org/TR/sparql11-query/)
- [SPARQL 1.1 Update](https://www.w3.org/TR/sparql11-update/)
- [Turtle - RDF 1.1](https://www.w3.org/TR/turtle/)
- [Oxigraph Documentation](https://github.com/oxigraph/oxigraph)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/store.R
\name{rdf_remove}
\alias{rdf_remove}
\title{Remove a Triple}
\usage{
rdf_remove(store, subject, predicate, object, graph = NULL)
}
\arguments{
\item{store}{An RDF store handle}

\item{subject}{Subject IRI or blank node}

\item{predicate}{Predicate IRI}

\item{object}{Object: IRI, blank node, or literal}

\item{graph}{Optional named graph IRI}
}
\value{
Invisibly returns NULL
}
\description{
Removes a single triple from the store.
}
\examples{
store <- rdf_store()
rdf_add(store, "<http://example.org/s>", "<http://example.org/p>", '"hello"')
rdf_remove(store, "<http://example.org/s>", "<http://example.org/p>", '"hello"')
rdf_size(store)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extendr-wrappers.R
\name{rdf_store_dump}
\alias{rdf_store_dump}
\title{Serialize the store contents to a string}
\usage{
rdf_store_dump(store_idx, format)
}
\arguments{
\item{store_idx}{Store index}

\item{format}{RDF format: "turtle", "ntriples", "rdfxml", "nquads", "trig"}
}
\value{
The serialized RDF data
}
\description{
Serialize the store contents to a string
}
\examples{
store <- rdf_store_new()
rdf_store_load(store, '<http://example.org/s> <http://example.org/p> "v" .', "ntriples", NULL)
rdf_store_dump(store, "turtle")
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/store.R
\name{sparql_query}
\alias{sparql_query}
\title{Execute a SPARQL Query}
\usage{
sparql_query(store, query)
}
\arguments{
\item{store}{An RDF store handle}

\item{query}{A SPARQL query string}
}
\value{
For SELECT queries, a data.frame with results. For ASK queries, a logical.
For CONSTRUCT/DESCRIBE queries, a data.frame with subject, predicate, object columns.
}
\description{
Executes a SPARQL query against the RDF store.
}
\examples{
store <- rdf_store()
rdf_load(store, '<http://example.org/s> <http://example.org/p> "hello" .', format = "ntriples")
sparql_query(store, "SELECT * WHERE { ?s ?p ?o }")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extendr-wrappers.R
\name{rdf_store_new}
\alias{rdf_store_new}
\title{Create a new in-memory RDF store}
\usage{
rdf_store_new()
}
\value{
Store index (integer handle)
}
\description{
Create a new in-memory RDF store
}
\section{Arguments}{

This function takes no arguments.
}

\examples{
store <- rdf_store_new()
rdf_store_size(store)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extendr-wrappers.R
\name{rdf_store_size}
\alias{rdf_store_size}
\title{Get the number of quads in the store}
\usage{
rdf_store_size(store_idx)
}
\arguments{
\item{store_idx}{Store index}
}
\value{
The number of quads
}
\description{
Get the number of quads in the store
}
\examples{
store <- rdf_store_new()
rdf_store_size(store)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/store.R
\name{rdf_load}
\alias{rdf_load}
\title{Load RDF Data}
\usage{
rdf_load(store, data, format = "turtle", base_iri = NULL)
}
\arguments{
\item{store}{An RDF store handle}

\item{data}{RDF data as a character string}

\item{format}{RDF format: "turtle", "ntriples", "rdfxml", "nquads", or "trig"}

\item{base_iri}{Optional base IRI for resolving relative URIs}
}
\value{
Invisibly returns NULL
}
\description{
Loads RDF data into the store from a string.
}
\examples{
store <- rdf_store()
rdf_load(store, '<http://example.org/s> <http://example.org/p> "value" .', format = "ntriples")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extendr-wrappers.R
\name{rdf_store_insert}
\alias{rdf_store_insert}
\title{Insert a triple into the store}
\usage{
rdf_store_insert(store_idx, subject, predicate, object, graph)
}
\arguments{
\item{store_idx}{Store index}

\item{subject}{Subject IRI (e.g., \code{"<http://example.org/s>"}) or blank node ("_:b1")}

\item{predicate}{Predicate IRI (e.g., \code{"<http://example.org/p>"})}

\item{object}{Object (IRI, blank node, or literal with quotes e.g., "\"value\"")}

\item{graph}{Optional graph name IRI}
}
\value{
No return value, called for side effects (inserting triples into the store)
}
\description{
Insert a triple into the store
}
\examples{
store <- rdf_store_new()
rdf_store_insert(store, "<http://example.org/s>", "<http://example.org/p>", '"val"', NULL)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/store.R
\name{rdf_store}
\alias{rdf_store}
\title{Create an RDF Store}
\usage{
rdf_store(path = NULL)
}
\arguments{
\item{path}{Optional path for persistent storage. If NULL (default), creates
an in-memory store.}
}
\value{
An RDF store handle (integer)
}
\description{
Creates a new RDF store, either in-memory or backed by persistent storage.
}
\examples{
# In-memory store
store <- rdf_store()

# Persistent store (not supported on Windows)
\donttest{
if (.Platform$OS.type != "windows") {
  store <- rdf_store(file.path(tempdir(), "my_store"))
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/store.R
\name{rdf_serialize}
\alias{rdf_serialize}
\title{Serialize RDF Data}
\usage{
rdf_serialize(store, format = "turtle")
}
\arguments{
\item{store}{An RDF store handle}

\item{format}{RDF format: "turtle", "ntriples", "rdfxml", "nquads", or "trig"}
}
\value{
The serialized RDF data as a character string
}
\description{
Serializes the store contents to a string.
}
\examples{
store <- rdf_store()
rdf_load(store, '<http://example.org/s> <http://example.org/p> "value" .', format = "ntriples")
rdf_serialize(store, format = "turtle")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extendr-wrappers.R
\name{rdf_store_query}
\alias{rdf_store_query}
\title{Execute a SPARQL query and return results as a data frame}
\usage{
rdf_store_query(store_idx, query)
}
\arguments{
\item{store_idx}{Store index}

\item{query}{SPARQL query string}
}
\value{
Query results as a data frame (for SELECT) or logical (for ASK)
}
\description{
Execute a SPARQL query and return results as a data frame
}
\examples{
store <- rdf_store_new()
rdf_store_load(store, '<http://example.org/s> <http://example.org/p> "v" .', "ntriples", NULL)
rdf_store_query(store, "SELECT * WHERE { ?s ?p ?o }")
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/store.R
\name{sparql_update}
\alias{sparql_update}
\title{Execute SPARQL Update}
\usage{
sparql_update(store, update)
}
\arguments{
\item{store}{An RDF store handle}

\item{update}{A SPARQL UPDATE query string}
}
\value{
Invisibly returns NULL
}
\description{
Executes a SPARQL UPDATE query to modify the store.
}
\examples{
store <- rdf_store()
sparql_update(store, "INSERT DATA { <http://example.org/s> <http://example.org/p> 'value' }")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/roxigraph-package.R
\docType{package}
\name{roxigraph-package}
\alias{roxigraph}
\alias{roxigraph-package}
\title{roxigraph: RDF and SPARQL for R}
\description{
Provides RDF storage and SPARQL 1.1 query capabilities by wrapping the
'Oxigraph' graph database library. Supports in-memory and persistent ('RocksDB')
storage, multiple RDF serialization formats, and full SPARQL 1.1 Query and
Update support.
}
\section{Main Functions}{

\itemize{
\item \code{\link{rdf_store}}: Create an RDF store
\item \code{\link{rdf_load}}: Load RDF data
\item \code{\link{sparql_query}}: Execute SPARQL queries
\item \code{\link{sparql_update}}: Execute SPARQL updates
\item \code{\link{rdf_serialize}}: Serialize RDF data
}
}

\seealso{
Useful links:
\itemize{
\item \url{https://github.com/cboettig/roxigraph}
\item \url{https://github.com/oxigraph/oxigraph}
\item Report bugs at \url{https://github.com/cboettig/roxigraph/issues}
}
}
\author{
\strong{Maintainer}: Carl Boettiger \email{cboettig@gmail.com} (\href{https://orcid.org/0000-0002-1642-628X}{ORCID})

Authors:
\itemize{
  \item Authors of the dependency Rust crates (see inst/AUTHORS file)
}

Other contributors:
\itemize{
  \item Oxigraph Contributors (Oxigraph Rust library) [copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extendr-wrappers.R
\name{rdf_store_open}
\alias{rdf_store_open}
\title{Open or create a persistent RDF store at the given path}
\usage{
rdf_store_open(path)
}
\arguments{
\item{path}{Path to the store directory}
}
\value{
Store index (integer handle)
}
\description{
Open or create a persistent RDF store at the given path
}
\examples{
\donttest{
if (.Platform$OS.type != "windows") {
  store <- rdf_store_open(file.path(tempdir(), "roxigraph_test"))
}
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/store.R
\name{rdf_size}
\alias{rdf_size}
\title{Get Store Size}
\usage{
rdf_size(store)
}
\arguments{
\item{store}{An RDF store handle}
}
\value{
The number of quads as an integer
}
\description{
Returns the number of quads (triples) in the store.
}
\examples{
store <- rdf_store()
rdf_size(store)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extendr-wrappers.R
\name{rdf_store_update}
\alias{rdf_store_update}
\title{Execute a SPARQL UPDATE query}
\usage{
rdf_store_update(store_idx, update)
}
\arguments{
\item{store_idx}{Store index}

\item{update}{SPARQL UPDATE query string}
}
\value{
No return value, called for side effects (executing SPARQL UPDATE queries)
}
\description{
Execute a SPARQL UPDATE query
}
\examples{
store <- rdf_store_new()
rdf_store_update(store, "INSERT DATA { <http://example.org/s> <http://example.org/p> 'val' }")
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extendr-wrappers.R
\name{rdf_store_load}
\alias{rdf_store_load}
\title{Load RDF data into the store}
\usage{
rdf_store_load(store_idx, data, format, base_iri)
}
\arguments{
\item{store_idx}{Store index}

\item{data}{RDF data as a string}

\item{format}{RDF format: "turtle", "ntriples", "rdfxml", "nquads", "trig"}

\item{base_iri}{Optional base IRI for relative URIs}
}
\value{
No return value, called for side effects (loading data into the store)
}
\description{
Load RDF data into the store
}
\examples{
store <- rdf_store_new()
rdf_store_load(store, '<http://example.org/s> <http://example.org/p> "v" .', "ntriples", NULL)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/store.R
\name{rdf_load_file}
\alias{rdf_load_file}
\title{Load RDF from File}
\usage{
rdf_load_file(store, file, format = NULL, base_iri = NULL)
}
\arguments{
\item{store}{An RDF store handle}

\item{file}{Path to the RDF file}

\item{format}{RDF format. If NULL, guessed from file extension.}

\item{base_iri}{Optional base IRI for resolving relative URIs}
}
\value{
Invisibly returns NULL
}
\description{
Loads RDF data into the store from a file.
}
\examples{
store <- rdf_store()
# Create a temporary RDF file
tmp <- tempfile(fileext = ".nt")
writeLines('<http://example.org/s> <http://example.org/p> "value" .', tmp)
rdf_load_file(store, tmp)
rdf_size(store)
}
